@@ -15,7 +15,7 @@ from core import error
 from core.error import e_usage
 from core import state
 from core import vm
-from frontend import flag_spec
+from frontend import flag_util
 from frontend import args
 from mycpp import mylib
 from mycpp.mylib import log
@@ -256,7 +256,7 @@ class Export(vm._AssignBuiltin):
         # type: (cmd_value.Assign) -> int
         arg_r = args.Reader(cmd_val.argv, locs=cmd_val.arg_locs)
         arg_r.Next()
-        attrs = flag_spec.Parse('export_', arg_r)
+        attrs = flag_util.Parse('export_', arg_r)
         arg = arg_types.export_(attrs.attrs)
         #arg = attrs
 
@@ -327,7 +327,7 @@ class Readonly(vm._AssignBuiltin):
         # type: (cmd_value.Assign) -> int
         arg_r = args.Reader(cmd_val.argv, locs=cmd_val.arg_locs)
         arg_r.Next()
-        attrs = flag_spec.Parse('readonly', arg_r)
+        attrs = flag_util.Parse('readonly', arg_r)
         arg = arg_types.readonly(attrs.attrs)
 
         if arg.p or len(cmd_val.pairs) == 0:
@@ -383,7 +383,7 @@ class NewVar(vm._AssignBuiltin):
         # type: (cmd_value.Assign) -> int
         arg_r = args.Reader(cmd_val.argv, locs=cmd_val.arg_locs)
         arg_r.Next()
-        attrs = flag_spec.Parse('new_var', arg_r)
+        attrs = flag_util.Parse('new_var', arg_r)
         arg = arg_types.new_var(attrs.attrs)
 
         status = 0
@@ -519,7 +519,7 @@ class Unset(vm._Builtin):
 
     def Run(self, cmd_val):
         # type: (cmd_value.Argv) -> int
-        attrs, arg_r = flag_spec.ParseCmdVal('unset', cmd_val)
+        attrs, arg_r = flag_util.ParseCmdVal('unset', cmd_val)
         arg = arg_types.unset(attrs.attrs)
 
         argv, arg_locs = arg_r.Rest2()
