@@ -12,6 +12,7 @@ from _devbuild.gen.value_asdl import (value, value_e, value_t, sh_lvalue,
 
 from core import error
 from core import optview
+from core import num
 from core import state
 from core import ui
 from data_lang import j8
@@ -68,11 +69,10 @@ class CrashDumper(object):
     One is constant at build time; the other is constant at runtime.
     """
 
-    def __init__(self, crash_dump_dir, fd_state, j8print):
-        # type: (str, process.FdState, j8.Printer) -> None
+    def __init__(self, crash_dump_dir, fd_state):
+        # type: (str, process.FdState) -> None
         self.crash_dump_dir = crash_dump_dir
         self.fd_state = fd_state
-        self.j8print = j8print
 
         # whether we should collect a dump, at the highest level of the stack
         self.do_collect = bool(crash_dump_dir)
@@ -106,7 +106,7 @@ class CrashDumper(object):
             # render that.
             self.error['source'] = value.Str(
                 ui.GetLineSourceString(blame_tok.line))
-            self.error['line_num'] = value.Int(blame_tok.line.line_num)
+            self.error['line_num'] = num.ToBig(blame_tok.line.line_num)
             self.error['line'] = value.Str(blame_tok.line.content)
 
         # TODO: Collect functions, aliases, etc.
@@ -142,8 +142,8 @@ class CrashDumper(object):
             'argv_stack': value.List(self.argv_stack),
             'debug_stack': value.List(self.debug_stack),
             'error': value.Dict(self.error),
-            'status': value.Int(status),
-            'pid': value.Int(my_pid),
+            'status': num.ToBig(status),
+            'pid': num.ToBig(my_pid),
         }  # type: Dict[str, value_t]
 
         path = os_path.join(self.crash_dump_dir,
@@ -151,7 +151,7 @@ class CrashDumper(object):
 
         # TODO: This should be JSON with unicode replacement char?
         buf = mylib.BufWriter()
-        self.j8print.PrintMessage(value.Dict(d), buf, 2)
+        j8.PrintMessage(value.Dict(d), buf, 2)
         json_str = buf.getvalue()
 
         try:
@@ -230,6 +230,14 @@ def _PrintShValue(val, buf):
     buf.write(result)
 
 
+def PrintShellArgv(argv, buf):
+    # type: (List[str], mylib.BufWriter) -> None
+    for i, arg in enumerate(argv):
+        if i != 0:
+            buf.write(' ')
+        buf.write(j8_lite.MaybeShellEncode(arg))
+
+
 def _PrintYshArgv(argv, buf):
     # type: (List[str], mylib.BufWriter) -> None
 
@@ -263,7 +271,7 @@ class Tracer(object):
     Other hooks:
 
     - Command completion starts other processes
-    - Oil command constructs: BareDecl, VarDecl, Mutation, Expr
+    - YSH command constructs: BareDecl, VarDecl, Mutation, Expr
     """
 
     def __init__(
@@ -276,12 +284,12 @@ class Tracer(object):
     ):
         # type: (...) -> None
         """
-    Args:
-      parse_ctx: For parsing PS4.
-      exec_opts: For xtrace setting
-      mem: for retrieving PS4
-      word_ev: for evaluating PS4
-    """
+        Args:
+          parse_ctx: For parsing PS4.
+          exec_opts: For xtrace setting
+          mem: for retrieving PS4
+          word_ev: for evaluating PS4
+        """
         self.parse_ctx = parse_ctx
         self.exec_opts = exec_opts
         self.mutable_opts = mutable_opts
@@ -299,6 +307,9 @@ class Tracer(object):
         # Mutate objects to save allocations
         self.val_indent = value.Str('')
         self.val_punct = value.Str('')
+        # TODO: show something for root process by default?  INTERLEAVED output
+        # can be confusing, e.g. debugging traps in forkred subinterpreter
+        # created by a pipeline.
         self.val_pid_str = value.Str('')  # mutated by SetProcess
 
         # Can these be global constants?  I don't think we have that in ASDL yet.
@@ -479,9 +490,9 @@ class Tracer(object):
             buf.write('\n')
             self.f.write(buf.getvalue())
 
-    def PrintMessage(self, message):
+    def OtherMessage(self, message):
         # type: (str) -> None
-        """Used when receiving signals."""
+        """Can be used when receiving signals."""
         buf = self._RichTraceBegin('!')
         if not buf:
             return
@@ -530,10 +541,7 @@ class Tracer(object):
             return
 
         # Legacy: Use SHELL encoding, NOT _PrintYshArgv()
-        for i, arg in enumerate(argv):
-            if i != 0:
-                buf.write(' ')
-            buf.write(j8_lite.MaybeShellEncode(arg))
+        PrintShellArgv(argv, buf)
         buf.write('\n')
         self.f.write(buf.getvalue())
 
