@@ -1,4 +1,5 @@
 from asdl import pybase
+from mycpp import mops
 from typing import Optional, List, Tuple, Dict, Any, cast, TYPE_CHECKING
 
 if TYPE_CHECKING:
@@ -59,7 +60,7 @@ class cmd_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('cmd_value.Argv')
       L = out_node.fields
@@ -105,7 +106,7 @@ class cmd_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('cmd_value.Argv')
       L = out_node.fields
@@ -170,7 +171,7 @@ class cmd_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('cmd_value.Assign')
       L = out_node.fields
@@ -207,7 +208,7 @@ class cmd_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('cmd_value.Assign')
       L = out_node.fields
@@ -290,7 +291,7 @@ class part_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('part_value.String')
       L = out_node.fields
@@ -311,7 +312,7 @@ class part_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('part_value.String')
       L = out_node.fields
@@ -348,7 +349,7 @@ class part_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('part_value.Array')
       L = out_node.fields
@@ -366,7 +367,7 @@ class part_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('part_value.Array')
       L = out_node.fields
@@ -400,7 +401,7 @@ class part_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('part_value.ExtGlob')
       L = out_node.fields
@@ -420,7 +421,7 @@ class part_value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('part_value.ExtGlob')
       L = out_node.fields
@@ -522,7 +523,7 @@ class a_index(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('a_index.Str')
       L = out_node.fields
@@ -537,7 +538,7 @@ class a_index(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('a_index.Str')
       L = out_node.fields
@@ -568,7 +569,7 @@ class a_index(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('a_index.Int')
       L = out_node.fields
@@ -583,7 +584,7 @@ class a_index(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('a_index.Int')
       L = out_node.fields
@@ -640,7 +641,7 @@ class redirect_arg__CloseFd(redirect_arg_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('redirect_arg__CloseFd')
     L = out_node.fields
@@ -652,7 +653,7 @@ class redirect_arg__CloseFd(redirect_arg_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('redirect_arg__CloseFd')
     L = out_node.fields
@@ -681,7 +682,7 @@ class redirect_arg(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('redirect_arg.Path')
       L = out_node.fields
@@ -696,7 +697,7 @@ class redirect_arg(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('redirect_arg.Path')
       L = out_node.fields
@@ -727,7 +728,7 @@ class redirect_arg(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('redirect_arg.CopyFd')
       L = out_node.fields
@@ -742,7 +743,7 @@ class redirect_arg(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('redirect_arg.CopyFd')
       L = out_node.fields
@@ -773,7 +774,7 @@ class redirect_arg(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('redirect_arg.MoveFd')
       L = out_node.fields
@@ -788,7 +789,7 @@ class redirect_arg(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('redirect_arg.MoveFd')
       L = out_node.fields
@@ -821,7 +822,7 @@ class redirect_arg(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('redirect_arg.HereDoc')
       L = out_node.fields
@@ -836,7 +837,7 @@ class redirect_arg(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('redirect_arg.HereDoc')
       L = out_node.fields
@@ -895,7 +896,7 @@ class wait_status(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('wait_status.Proc')
       L = out_node.fields
@@ -910,7 +911,7 @@ class wait_status(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('wait_status.Proc')
       L = out_node.fields
@@ -941,7 +942,7 @@ class wait_status(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('wait_status.Pipeline')
       L = out_node.fields
@@ -959,7 +960,7 @@ class wait_status(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('wait_status.Pipeline')
       L = out_node.fields
@@ -993,7 +994,7 @@ class wait_status(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('wait_status.Cancelled')
       L = out_node.fields
@@ -1008,7 +1009,7 @@ class wait_status(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('wait_status.Cancelled')
       L = out_node.fields
@@ -1215,7 +1216,7 @@ class trace__CommandSub(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__CommandSub')
     L = out_node.fields
@@ -1227,7 +1228,7 @@ class trace__CommandSub(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__CommandSub')
     L = out_node.fields
@@ -1250,7 +1251,7 @@ class trace__ForkWait(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__ForkWait')
     L = out_node.fields
@@ -1262,7 +1263,7 @@ class trace__ForkWait(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__ForkWait')
     L = out_node.fields
@@ -1285,7 +1286,7 @@ class trace__Fork(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__Fork')
     L = out_node.fields
@@ -1297,7 +1298,7 @@ class trace__Fork(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__Fork')
     L = out_node.fields
@@ -1320,7 +1321,7 @@ class trace__PipelinePart(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__PipelinePart')
     L = out_node.fields
@@ -1332,7 +1333,7 @@ class trace__PipelinePart(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__PipelinePart')
     L = out_node.fields
@@ -1355,7 +1356,7 @@ class trace__ProcessSub(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__ProcessSub')
     L = out_node.fields
@@ -1367,7 +1368,7 @@ class trace__ProcessSub(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__ProcessSub')
     L = out_node.fields
@@ -1390,7 +1391,7 @@ class trace__HereDoc(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__HereDoc')
     L = out_node.fields
@@ -1402,7 +1403,7 @@ class trace__HereDoc(trace_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('trace__HereDoc')
     L = out_node.fields
@@ -1431,7 +1432,7 @@ class trace(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('trace.External')
       L = out_node.fields
@@ -1449,7 +1450,7 @@ class trace(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('trace.External')
       L = out_node.fields
@@ -1538,7 +1539,7 @@ class AssignArg(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('AssignArg')
     L = out_node.fields
@@ -1564,7 +1565,7 @@ class AssignArg(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('AssignArg')
     L = out_node.fields
@@ -1607,7 +1608,7 @@ class VarSubState(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('VarSubState')
     L = out_node.fields
@@ -1625,7 +1626,7 @@ class VarSubState(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('VarSubState')
     L = out_node.fields
@@ -1662,7 +1663,7 @@ class Cell(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('Cell')
     L = out_node.fields
@@ -1687,7 +1688,7 @@ class Cell(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('Cell')
     L = out_node.fields
@@ -1729,7 +1730,7 @@ class VTestPlace(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('VTestPlace')
     L = out_node.fields
@@ -1749,7 +1750,7 @@ class VTestPlace(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('VTestPlace')
     L = out_node.fields
@@ -1788,7 +1789,7 @@ class RedirValue(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('RedirValue')
     L = out_node.fields
@@ -1815,7 +1816,7 @@ class RedirValue(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('RedirValue')
     L = out_node.fields
@@ -1859,7 +1860,7 @@ class StatusArray(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('StatusArray')
     L = out_node.fields
@@ -1885,7 +1886,7 @@ class StatusArray(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('StatusArray')
     L = out_node.fields
@@ -1933,7 +1934,7 @@ class CommandStatus(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('CommandStatus')
     L = out_node.fields
@@ -1968,7 +1969,7 @@ class CommandStatus(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('CommandStatus')
     L = out_node.fields
@@ -2019,7 +2020,7 @@ class HayNode(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('HayNode')
     L = out_node.fields
@@ -2039,7 +2040,7 @@ class HayNode(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('HayNode')
     L = out_node.fields
