@@ -1,4 +1,5 @@
 from asdl import pybase
+from mycpp import mops
 from typing import Optional, List, Tuple, Dict, Any, cast, TYPE_CHECKING
 
 if TYPE_CHECKING:
@@ -53,7 +54,7 @@ class y_lvalue(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('y_lvalue.Container')
       L = out_node.fields
@@ -73,7 +74,7 @@ class y_lvalue(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('y_lvalue.Container')
       L = out_node.fields
@@ -139,7 +140,7 @@ class sh_lvalue(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('sh_lvalue.Indexed')
       L = out_node.fields
@@ -161,7 +162,7 @@ class sh_lvalue(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('sh_lvalue.Indexed')
       L = out_node.fields
@@ -201,7 +202,7 @@ class sh_lvalue(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('sh_lvalue.Keyed')
       L = out_node.fields
@@ -223,7 +224,7 @@ class sh_lvalue(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('sh_lvalue.Keyed')
       L = out_node.fields
@@ -281,7 +282,7 @@ class eggex_ops__No(eggex_ops_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('eggex_ops__No')
     L = out_node.fields
@@ -293,7 +294,7 @@ class eggex_ops__No(eggex_ops_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('eggex_ops__No')
     L = out_node.fields
@@ -326,7 +327,7 @@ class eggex_ops(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('eggex_ops.Yes')
       L = out_node.fields
@@ -360,7 +361,7 @@ class eggex_ops(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('eggex_ops.Yes')
       L = out_node.fields
@@ -430,7 +431,7 @@ class regex_match__No(regex_match_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('regex_match__No')
     L = out_node.fields
@@ -442,7 +443,7 @@ class regex_match__No(regex_match_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('regex_match__No')
     L = out_node.fields
@@ -541,7 +542,7 @@ class value__Undef(value_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('value__Undef')
     L = out_node.fields
@@ -553,7 +554,7 @@ class value__Undef(value_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('value__Undef')
     L = out_node.fields
@@ -576,7 +577,7 @@ class value__Null(value_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('value__Null')
     L = out_node.fields
@@ -588,7 +589,7 @@ class value__Null(value_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('value__Null')
     L = out_node.fields
@@ -619,7 +620,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Str')
       L = out_node.fields
@@ -634,7 +635,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Str')
       L = out_node.fields
@@ -665,7 +666,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.BashArray')
       L = out_node.fields
@@ -683,7 +684,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.BashArray')
       L = out_node.fields
@@ -717,7 +718,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.BashAssoc')
       L = out_node.fields
@@ -737,7 +738,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.BashAssoc')
       L = out_node.fields
@@ -775,7 +776,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Bool')
       L = out_node.fields
@@ -790,7 +791,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Bool')
       L = out_node.fields
@@ -808,25 +809,25 @@ class value(object):
     __slots__ = ('i',)
   
     def __init__(self, i):
-      # type: (int) -> None
+      # type: (mops.BigInt) -> None
       self.i = i
   
     @staticmethod
     def CreateNull(alloc_lists=False):
       # type: () -> value.Int
-      return value.Int(-1)
+      return value.Int(mops.BigInt(-1))
   
     def PrettyTree(self, trav=None):
       # type: (Optional[TraversalState]) -> hnode_t
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Int')
       L = out_node.fields
   
-      x0 = hnode.Leaf(str(self.i), color_e.OtherConst)
+      x0 = hnode.Leaf(mops.ToStr(self.i), color_e.OtherConst)
       L.append(Field('i', x0))
   
       return out_node
@@ -836,11 +837,11 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Int')
       L = out_node.fields
-      x0 = hnode.Leaf(str(self.i), color_e.OtherConst)
+      x0 = hnode.Leaf(mops.ToStr(self.i), color_e.OtherConst)
       L.append(Field('i', x0))
   
       return out_node
@@ -867,7 +868,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Float')
       L = out_node.fields
@@ -882,7 +883,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Float')
       L = out_node.fields
@@ -913,7 +914,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.List')
       L = out_node.fields
@@ -933,7 +934,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.List')
       L = out_node.fields
@@ -969,7 +970,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Dict')
       L = out_node.fields
@@ -989,7 +990,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Dict')
       L = out_node.fields
@@ -1032,7 +1033,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Eggex')
       L = out_node.fields
@@ -1077,7 +1078,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Eggex')
       L = out_node.fields
@@ -1138,7 +1139,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Expr')
       L = out_node.fields
@@ -1154,7 +1155,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Expr')
       L = out_node.fields
@@ -1186,7 +1187,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Template')
       L = out_node.fields
@@ -1202,7 +1203,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Template')
       L = out_node.fields
@@ -1234,7 +1235,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Command')
       L = out_node.fields
@@ -1250,7 +1251,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Command')
       L = out_node.fields
@@ -1282,7 +1283,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Block')
       L = out_node.fields
@@ -1298,7 +1299,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Block')
       L = out_node.fields
@@ -1331,7 +1332,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Place')
       L = out_node.fields
@@ -1355,7 +1356,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Place')
       L = out_node.fields
@@ -1395,7 +1396,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Module')
       L = out_node.fields
@@ -1415,7 +1416,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Module')
       L = out_node.fields
@@ -1452,7 +1453,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.IO')
       L = out_node.fields
@@ -1470,7 +1471,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.IO')
       L = out_node.fields
@@ -1504,7 +1505,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Guts')
       L = out_node.fields
@@ -1519,7 +1520,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Guts')
       L = out_node.fields
@@ -1550,7 +1551,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.BuiltinFunc')
       L = out_node.fields
@@ -1565,7 +1566,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.BuiltinFunc')
       L = out_node.fields
@@ -1597,7 +1598,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.BoundFunc')
       L = out_node.fields
@@ -1617,7 +1618,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.BoundFunc')
       L = out_node.fields
@@ -1658,7 +1659,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Proc')
       L = out_node.fields
@@ -1692,7 +1693,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Proc')
       L = out_node.fields
@@ -1746,7 +1747,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Func')
       L = out_node.fields
@@ -1789,7 +1790,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Func')
       L = out_node.fields
@@ -1849,7 +1850,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Slice')
       L = out_node.fields
@@ -1869,7 +1870,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Slice')
       L = out_node.fields
@@ -1906,7 +1907,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Range')
       L = out_node.fields
@@ -1924,7 +1925,7 @@ class value(object):
       trav = trav or TraversalState()
       heap_id = id(self)
       if heap_id in trav.seen:
-        return hnode.ObjectCycle(heap_id)
+        return hnode.AlreadySeen(heap_id)
       trav.seen[heap_id] = True
       out_node = NewRecord('value.Range')
       L = out_node.fields
@@ -1960,7 +1961,7 @@ class IntBox(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('IntBox')
     L = out_node.fields
@@ -1975,7 +1976,7 @@ class IntBox(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('IntBox')
     L = out_node.fields
@@ -2008,7 +2009,7 @@ class ProcDefaults(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('ProcDefaults')
     L = out_node.fields
@@ -2044,7 +2045,7 @@ class ProcDefaults(pybase.CompoundObj):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('ProcDefaults')
     L = out_node.fields
@@ -2097,7 +2098,7 @@ class LeftName(y_lvalue_t, sh_lvalue_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('LeftName')
     L = out_node.fields
@@ -2116,7 +2117,7 @@ class LeftName(y_lvalue_t, sh_lvalue_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('LeftName')
     L = out_node.fields
@@ -2153,7 +2154,7 @@ class RegexMatch(regex_match_t, value_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('RegexMatch')
     L = out_node.fields
@@ -2178,7 +2179,7 @@ class RegexMatch(regex_match_t, value_t):
     trav = trav or TraversalState()
     heap_id = id(self)
     if heap_id in trav.seen:
-      return hnode.ObjectCycle(heap_id)
+      return hnode.AlreadySeen(heap_id)
     trav.seen[heap_id] = True
     out_node = NewRecord('RegexMatch')
     L = out_node.fields
