@@ -14,35 +14,36 @@ Later:
   - line wrapping -- do this later
   - would like CONTRIBUTORS here
 
-- Unify with ASDL pretty printing - TYG8
+- Unify with ASDL pretty printing - NIL8
    - {} [] are identical
    - () is for statically typed ASDL data
      (command.Simple blame_tok:(...) words:[ ])
      although we are also using [] for typed ASDL arrays, not just JSON
    - object IDs
-     - @0x123 can create an ID
-     - *0x123 can reference an ID
-       - or maybe <0x123> though that's longer
+     - @ x123 can create an ID
+     - ! x123 can reference an ID
    - <> can be for non-J8 data types?  For the = operator
    - 'hi \(name)' interpolation is useful for code
 
-- Common between JSON8 and TYG8 - for writing by hand
+- Common between JSON8 and NIL8 - for writing by hand
   - comments - # line or // line (JSON5 uses // line, following JS)
   - unquoted identifier names - TYG8 could be more relaxed for (+ 1 (* 3 4))
   - commas
     - JSON8 could have trailing commas rule
-    - TYG8 at least has no commas for [1 2 "hi"]
+    - NIL8 at least has no commas for [1 2 "hi"]
 """
 
 from _devbuild.gen.id_kind_asdl import Id, Id_t, Id_str
-from _devbuild.gen.value_asdl import (value, value_e, value_t)
+from _devbuild.gen.value_asdl import (value, value_e, value_t, value_str)
+from _devbuild.gen.nil8_asdl import (nvalue, nvalue_t)
 
 from asdl import format as fmt
 from core import error
-from core import vm
 from data_lang import pyj8
+# dependency issue: consts.py pulls in frontend/option_def.py
 from frontend import consts
 from frontend import match
+from mycpp import mops
 from mycpp import mylib
 from mycpp.mylib import tagswitch, iteritems, NewDict, log
 
@@ -52,6 +53,95 @@ _ = log
 
 from typing import cast, Dict, List, Tuple, Optional
 
+
+# COPIED from ui.ValType() to break dep
+def ValType(val):
+    # type: (value_t) -> str
+    """For displaying type errors in the UI."""
+
+    return value_str(val.tag(), dot=False)
+
+
+if mylib.PYTHON:
+
+    def HeapValueId(val):
+        # type: (value_t) -> int
+        """
+        Python's id() returns the address, which is up to 64 bits.
+
+        In C++ we can use the GC ID, which fits within 32 bits.
+        """
+        return id(val)
+
+
+def ValueId(val):
+    # type: (value_t) -> int
+    """
+    Return an integer ID for object that:
+
+    1. Can be used to determine whether 2 objects are the same, e.g. for
+       List, Dict, Func, Proc, etc.
+    2. Will help detect object cycles
+
+    Primitives types like Int and Float don't have this notion.  They're
+    immutable values that are copied and compared by value.
+    """
+    with tagswitch(val) as case:
+        if case(value_e.Null, value_e.Bool, value_e.Int, value_e.Float,
+                value_e.Str):
+            # These will not be on the heap if we switch to tagged pointers
+            # Str is handled conservatively - when we add small string
+            # optimization, some strings will be values, so we assume all are.
+            return -1
+        else:
+            return HeapValueId(val)
+
+
+def ValueIdString(val):
+    # type: (value_t) -> str
+    """Used by pp value (42) and = 42"""
+    heap_id = ValueId(val)  # could be -1
+    if heap_id == -1:
+        return ''
+    else:
+        return ' 0x%s' % mylib.hex_lower(heap_id)
+
+
+def Utf8Encode(code):
+    # type: (int) -> str
+    """Return utf-8 encoded bytes from a unicode code point.
+
+    Based on https://stackoverflow.com/a/23502707
+    """
+    num_cont_bytes = 0
+
+    if code <= 0x7F:
+        return chr(code & 0x7F)  # ASCII
+
+    elif code <= 0x7FF:
+        num_cont_bytes = 1
+    elif code <= 0xFFFF:
+        num_cont_bytes = 2
+    elif code <= 0x10FFFF:
+        num_cont_bytes = 3
+
+    else:
+        return '\xEF\xBF\xBD'  # unicode replacement character
+
+    bytes_ = []  # type: List[int]
+    for _ in xrange(num_cont_bytes):
+        bytes_.append(0x80 | (code & 0x3F))
+        code >>= 6
+
+    b = (0x1E << (6 - num_cont_bytes)) | (code & (0x3F >> num_cont_bytes))
+    bytes_.append(b)
+    bytes_.reverse()
+
+    # mod 256 because Python ints don't wrap around!
+    tmp = [chr(b & 0xFF) for b in bytes_]
+    return ''.join(tmp)
+
+
 SHOW_CYCLES = 1 << 1  # show as [...] or {...} I think, with object ID
 SHOW_NON_DATA = 1 << 2  # non-data objects like Eggex can be <Eggex 0xff>
 LOSSY_JSON = 1 << 3  # JSON is lossy
@@ -60,95 +150,78 @@ LOSSY_JSON = 1 << 3  # JSON is lossy
 assert pyj8.LOSSY_JSON == LOSSY_JSON
 
 
-class Printer(object):
+def _Print(val, buf, indent, options=0):
+    # type: (value_t, mylib.BufWriter, int, int) -> None
     """
-    For json/j8 write (x), write (x), = operator, pp line (x)
+    Args:
+      indent: number of spaces to indent, or -1 for everything on one line
     """
+    p = InstancePrinter(buf, indent, options)
+    p.Print(val)
 
-    def __init__(self):
-        # type: () -> None
 
-        # TODO: should remove this in favor of BufWriter method
-        self.spaces = {0: ''}  # cache of strings with spaces
+def PrintMessage(val, buf, indent):
+    # type: (value_t, mylib.BufWriter, int) -> None
+    """ For json8 write (x) and toJson8() 
 
-    # Could be PrintMessage or PrintJsonMessage()
-    def _Print(self, val, buf, indent, options=0):
-        # type: (value_t, mylib.BufWriter, int, int) -> None
-        """
-        Args:
-          indent: number of spaces to indent, or -1 for everything on one line
-        """
-        p = InstancePrinter(buf, indent, options, self.spaces)
-        p.Print(val)
+    Caller must handle error.Encode
+    """
+    _Print(val, buf, indent)
 
-    def PrintMessage(self, val, buf, indent):
-        # type: (value_t, mylib.BufWriter, int) -> None
-        """ For j8 write (x) and toJ8() 
 
-        Caller must handle error.Encode
-        """
-        self._Print(val, buf, indent)
+def PrintJsonMessage(val, buf, indent):
+    # type: (value_t, mylib.BufWriter, int) -> None
+    """ For json write (x) and toJson()
 
-    def PrintJsonMessage(self, val, buf, indent):
-        # type: (value_t, mylib.BufWriter, int) -> None
-        """ For json write (x) and toJson()
+    Caller must handle error.Encode()
+    Doesn't decay to b'' strings - will use Unicode replacement char.
+    """
+    _Print(val, buf, indent, options=LOSSY_JSON)
 
-        Caller must handle error.Encode()
-        Doesn't decay to b'' strings - will use Unicode replacement char.
-        """
-        self._Print(val, buf, indent, options=LOSSY_JSON)
 
-    def DebugPrint(self, val, f):
-        # type: (value_t, mylib.Writer) -> None
-        """
-        For = operator.
-        """
-        # error.Encode should be impossible - we show cycles and non-data
-        buf = mylib.BufWriter()
-        self._Print(val, buf, -1, options=SHOW_CYCLES | SHOW_NON_DATA)
-        f.write(buf.getvalue())
-        f.write('\n')
+def PrintLine(val, f):
+    # type: (value_t, mylib.Writer) -> None
+    """ For pp line (x) """
 
-    def PrintLine(self, val, f):
-        # type: (value_t, mylib.Writer) -> None
-        """ For pp line (x) """
+    # error.Encode should be impossible - we show cycles and non-data
+    buf = mylib.BufWriter()
+    _Print(val, buf, -1, options=SHOW_CYCLES | SHOW_NON_DATA)
+    f.write(buf.getvalue())
+    f.write('\n')
 
-        # error.Encode should be impossible - we show cycles and non-data
-        buf = mylib.BufWriter()
-        self._Print(val, buf, -1, options=SHOW_CYCLES | SHOW_NON_DATA)
-        f.write(buf.getvalue())
-        f.write('\n')
 
-    def EncodeString(self, s, buf, unquoted_ok=False):
-        # type: (str, mylib.BufWriter, bool) -> None
-        """ For pp proc, etc."""
+def EncodeString(s, buf, unquoted_ok=False):
+    # type: (str, mylib.BufWriter, bool) -> None
+    """ For pp proc, etc."""
 
-        if unquoted_ok and fastfunc.CanOmitQuotes(s):
-            buf.write(s)
-            return
+    if unquoted_ok and fastfunc.CanOmitQuotes(s):
+        buf.write(s)
+        return
 
-        self._Print(value.Str(s), buf, -1)
+    _Print(value.Str(s), buf, -1)
 
-    def MaybeEncodeString(self, s):
-        # type: (str) -> str
-        """ For write --j8 $s  and compexport """
 
-        # TODO: add unquoted_ok here?
-        # /usr/local/foo-bar/x.y/a_b
+def MaybeEncodeString(s):
+    # type: (str) -> str
+    """ For write --json8 $s  and compexport """
 
-        buf = mylib.BufWriter()
-        self._Print(value.Str(s), buf, -1)
-        return buf.getvalue()
+    # TODO: add unquoted_ok here?
+    # /usr/local/foo-bar/x.y/a_b
 
-    def MaybeEncodeJsonString(self, s):
-        # type: (str) -> str
-        """ For write --json """
+    buf = mylib.BufWriter()
+    _Print(value.Str(s), buf, -1)
+    return buf.getvalue()
 
-        # TODO: add unquoted_ok here?
-        # /usr/local/foo-bar/x.y/a_b
-        buf = mylib.BufWriter()
-        self._Print(value.Str(s), buf, -1, options=LOSSY_JSON)
-        return buf.getvalue()
+
+def MaybeEncodeJsonString(s):
+    # type: (str) -> str
+    """ For write --json """
+
+    # TODO: add unquoted_ok here?
+    # /usr/local/foo-bar/x.y/a_b
+    buf = mylib.BufWriter()
+    _Print(value.Str(s), buf, -1, options=LOSSY_JSON)
+    return buf.getvalue()
 
 
 # DFS traversal state
@@ -156,94 +229,96 @@ UNSEEN = 0
 EXPLORING = 1
 FINISHED = 2
 
+
 class InstancePrinter(object):
     """Print a value tree as J8/JSON."""
 
-    def __init__(self, buf, indent, options, spaces):
-        # type: (mylib.BufWriter, int, int, Dict[int, str]) -> None
+    def __init__(self, buf, indent, options):
+        # type: (mylib.BufWriter, int, int) -> None
         self.buf = buf
         self.indent = indent
         self.options = options
-        self.spaces = spaces
 
         # Key is vm.HeapValueId(val)
         # Value is always True
         # Dict[int, None] doesn't translate -- it would be nice to have a set()
         self.visited = {}  # type: Dict[int, int]
 
-    def _GetIndent(self, num_spaces):
-        # type: (int) -> str
-        if num_spaces not in self.spaces:
-            self.spaces[num_spaces] = ' ' * num_spaces
-        return self.spaces[num_spaces]
+    def _ItemIndent(self, level):
+        # type: (int) -> None
+
+        if self.indent == -1:
+            return
+
+        self.buf.write_spaces((level + 1) * self.indent)
+
+    def _BracketIndent(self, level):
+        # type: (int) -> None
+
+        if self.indent == -1:
+            return
+
+        self.buf.write_spaces(level * self.indent)
+
+    def _MaybeNewline(self):
+        # type: () -> None
+        if self.indent == -1:
+            return
+        self.buf.write('\n')
+
+    def _MaybeSpace(self):
+        # type: () -> None
+        if self.indent == -1:
+            return
+        self.buf.write(' ')
 
     def _PrintList(self, val, level):
         # type: (value.List, int) -> None
 
-        if self.indent == -1:
-            bracket_indent = ''
-            item_indent = ''
-            maybe_newline = ''
-        else:
-            bracket_indent = self._GetIndent(level * self.indent)
-            item_indent = self._GetIndent((level + 1) * self.indent)
-            maybe_newline = '\n'
-
         if len(val.items) == 0:  # Special case like Python/JS
             self.buf.write('[]')
         else:
             self.buf.write('[')
-            self.buf.write(maybe_newline)
+            self._MaybeNewline()
             for i, item in enumerate(val.items):
                 if i != 0:
                     self.buf.write(',')
-                    self.buf.write(maybe_newline)
+                    self._MaybeNewline()
 
-                self.buf.write(item_indent)
+                self._ItemIndent(level)
                 self.Print(item, level + 1)
-            self.buf.write(maybe_newline)
+            self._MaybeNewline()
 
-            self.buf.write(bracket_indent)
+            self._BracketIndent(level)
             self.buf.write(']')
 
     def _PrintDict(self, val, level):
         # type: (value.Dict, int) -> None
 
-        if self.indent == -1:
-            bracket_indent = ''
-            item_indent = ''
-            maybe_newline = ''
-            maybe_space = ''
-        else:
-            bracket_indent = self._GetIndent(level * self.indent)
-            item_indent = self._GetIndent((level + 1) * self.indent)
-            maybe_newline = '\n'
-            maybe_space = ' '  # after colon
-
         if len(val.d) == 0:  # Special case like Python/JS
             self.buf.write('{}')
         else:
             self.buf.write('{')
-            self.buf.write(maybe_newline)
+            self._MaybeNewline()
             i = 0
             for k, v in iteritems(val.d):
                 if i != 0:
                     self.buf.write(',')
-                    self.buf.write(maybe_newline)
+                    self._MaybeNewline()
 
-                self.buf.write(item_indent)
+                self._ItemIndent(level)
 
                 pyj8.WriteString(k, self.options, self.buf)
 
                 self.buf.write(':')
-                self.buf.write(maybe_space)
+                self._MaybeSpace()
 
                 self.Print(v, level + 1)
 
                 i += 1
 
-            self.buf.write(maybe_newline)
-            self.buf.write(bracket_indent)
+            self._MaybeNewline()
+            self._BracketIndent(level)
             self.buf.write('}')
 
     def Print(self, val, level=0):
@@ -253,16 +328,6 @@ class InstancePrinter(object):
         # It's like
         #    JSON.stringify(d, null, 0)
         # except we use -1, not 0.  0 can still have newlines.
-        if self.indent == -1:
-            bracket_indent = ''
-            item_indent = ''
-            maybe_newline = ''
-            maybe_space = ''
-        else:
-            bracket_indent = self._GetIndent(level * self.indent)
-            item_indent = self._GetIndent((level + 1) * self.indent)
-            maybe_newline = '\n'
-            maybe_space = ' '  # after colon
 
         UP_val = val
         with tagswitch(val) as case:
@@ -275,13 +340,20 @@ class InstancePrinter(object):
 
             elif case(value_e.Int):
                 val = cast(value.Int, UP_val)
-                # TODO: use pyj8.WriteInt(val.i, self.buf)
-                self.buf.write(str(val.i))
+                # TODO: avoid intermediate allocation with
+                # self.buf.WriteBigInt(val.i)
+                #
+                # Or maybe we need pyj8.WriteBigInt() because truly BigInt may
+                # be of arbitrary length, and will need a growth strategy.
+                # Although that is not very common, so we could allocate in
+                # that case.
+
+                self.buf.write(mops.ToStr(val.i))
 
             elif case(value_e.Float):
                 val = cast(value.Float, UP_val)
-
-                # TODO: use pyj8.WriteFloat(val.f, self.buf)
+                # TODO: avoid intrmediate allocation with
+                # self.buf.WriteFloat(val.f)
                 self.buf.write(str(val.f))
 
             elif case(value_e.Str):
@@ -293,7 +365,7 @@ class InstancePrinter(object):
                 val = cast(value.List, UP_val)
 
                 # Cycle detection, only for containers that can be in cycles
-                heap_id = vm.HeapValueId(val)
+                heap_id = HeapValueId(val)
 
                 node_state = self.visited.get(heap_id, UNSEEN)
                 if node_state == FINISHED:
@@ -304,13 +376,13 @@ class InstancePrinter(object):
                     return
                 if node_state == EXPLORING:
                     if self.options & SHOW_CYCLES:
-                        self.buf.write('[ -->%s ]' % vm.ValueIdString(val))
+                        self.buf.write('[ -->%s ]' % ValueIdString(val))
                         return
                     else:
                         # node.js prints which index closes the cycle
                         raise error.Encode(
                             "Can't encode List%s in object cycle" %
-                            vm.ValueIdString(val))
+                            ValueIdString(val))
 
                 self.visited[heap_id] = EXPLORING
                 self._PrintList(val, level)
@@ -320,7 +392,7 @@ class InstancePrinter(object):
                 val = cast(value.Dict, UP_val)
 
                 # Cycle detection, only for containers that can be in cycles
-                heap_id = vm.HeapValueId(val)
+                heap_id = HeapValueId(val)
 
                 node_state = self.visited.get(heap_id, UNSEEN)
                 if node_state == FINISHED:
@@ -331,13 +403,13 @@ class InstancePrinter(object):
                     return
                 if node_state == EXPLORING:
                     if self.options & SHOW_CYCLES:
-                        self.buf.write('{ -->%s }' % vm.ValueIdString(val))
+                        self.buf.write('{ -->%s }' % ValueIdString(val))
                         return
                     else:
                         # node.js prints which key closes the cycle
                         raise error.Encode(
                             "Can't encode Dict%s in object cycle" %
-                            vm.ValueIdString(val))
+                            ValueIdString(val))
 
                 self.visited[heap_id] = EXPLORING
                 self._PrintDict(val, level)
@@ -354,47 +426,47 @@ class InstancePrinter(object):
                 val = cast(value.BashArray, UP_val)
 
                 self.buf.write('[')
-                self.buf.write(maybe_newline)
+                self._MaybeNewline()
                 for i, s in enumerate(val.strs):
                     if i != 0:
                         self.buf.write(',')
-                        self.buf.write(maybe_newline)
+                        self._MaybeNewline()
 
-                    self.buf.write(item_indent)
+                    self._ItemIndent(level)
                     if s is None:
                         self.buf.write('null')
                     else:
                         pyj8.WriteString(s, self.options, self.buf)
 
-                self.buf.write(maybe_newline)
+                self._MaybeNewline()
 
-                self.buf.write(bracket_indent)
+                self._BracketIndent(level)
                 self.buf.write(']')
 
             elif case(value_e.BashAssoc):
                 val = cast(value.BashAssoc, UP_val)
 
                 self.buf.write('{')
-                self.buf.write(maybe_newline)
+                self._MaybeNewline()
                 i = 0
                 for k2, v2 in iteritems(val.d):
                     if i != 0:
                         self.buf.write(',')
-                        self.buf.write(maybe_newline)
+                        self._MaybeNewline()
 
-                    self.buf.write(item_indent)
+                    self._ItemIndent(level)
 
                     pyj8.WriteString(k2, self.options, self.buf)
 
                     self.buf.write(':')
-                    self.buf.write(maybe_space)
+                    self._MaybeSpace()
 
                     pyj8.WriteString(v2, self.options, self.buf)
 
                     i += 1
 
-                self.buf.write(maybe_newline)
-                self.buf.write(bracket_indent)
+                self._MaybeNewline()
+                self._BracketIndent(level)
                 self.buf.write('}')
 
             else:
@@ -402,14 +474,12 @@ class InstancePrinter(object):
                 if self.options & SHOW_NON_DATA:
                     # Similar to = operator, ui.DebugPrint()
                     # TODO: that prints value.Range in a special way
-                    from core import ui
-                    ysh_type = ui.ValType(val)
-                    id_str = vm.ValueIdString(val)
+                    ysh_type = ValType(val)
+                    id_str = ValueIdString(val)
                     self.buf.write('<%s%s>' % (ysh_type, id_str))
                 else:
-                    from core import ui  # TODO: break dep
                     raise error.Encode("Can't serialize object of type %s" %
-                                       ui.ValType(val))
+                                       ValType(val))
 
 
 class PrettyPrinter(object):
@@ -440,7 +510,7 @@ class PrettyPrinter(object):
 
         # This could be an optimized set an C++ bit set like
         # mark_sweep_heap.h, rather than a Dict
-        self.unique_objs = mylib.UniqueObjects()
+        #self.unique_objs = mylib.UniqueObjects()
 
         # first pass of object ID -> number of times references
 
@@ -477,7 +547,7 @@ class LexerDecoder(object):
         # type: (str, bool) -> None
         self.s = s
         self.is_j8 = is_j8
-        self.lang_str = "J8" if is_j8 else "JSON"
+        self.lang_str = "NIL8"
 
         self.pos = 0
         # Reuse this instance to save GC objects.  JSON objects could have
@@ -496,25 +566,27 @@ class LexerDecoder(object):
 
         tok_id, end_pos = match.MatchJ8Token(self.s, self.pos)
 
+        if not self.is_j8:
+            if tok_id in (Id.Left_BSingleQuote, Id.Left_USingleQuote):
+                raise self._Error(
+                    "Single quotes aren't part of JSON; you may want 'json8 read'",
+                    end_pos)
+            if tok_id == Id.Ignored_Comment:
+                raise self._Error(
+                    "Comments aren't part of JSON; you may want 'json8 read'",
+                    end_pos)
+
         # Non-string tokens like { } null etc.
-        if tok_id not in (Id.Left_DoubleQuote, Id.Left_USingleQuote,
-                          Id.Left_BSingleQuote):
-            self.pos = end_pos
-            return tok_id, end_pos, None
+        if tok_id in (Id.Left_DoubleQuote, Id.Left_BSingleQuote,
+                      Id.Left_USingleQuote):
+            return self._DecodeString(tok_id, end_pos)
 
-        if not self.is_j8 and tok_id != Id.Left_DoubleQuote:
-            raise self._Error(
-                "Single quotes aren't part of JSON; you may want 'json8 read'",
-                end_pos)
-
-        return self._DecodeString(tok_id, end_pos)
+        self.pos = end_pos
+        return tok_id, end_pos, None
 
     def _DecodeString(self, left_id, str_pos):
         # type: (Id_t, int) -> Tuple[Id_t, int, Optional[str]]
-        """ Returns a string tkoen an updates self.pos """
-
-        # TODO: break dep
-        from osh import string_ops
+        """ Returns a string token and updates self.pos """
 
         while True:
             if left_id == Id.Left_DoubleQuote:
@@ -578,7 +650,7 @@ class LexerDecoder(object):
                         r"\u{%s} escape is illegal because it's in the surrogate range"
                         % h, str_end)
 
-                part = string_ops.Utf8Encode(i)
+                part = Utf8Encode(i)
 
             elif tok_id == Id.Char_YHex:  # J8 only
                 h = self.s[str_pos + 2:str_end]
@@ -602,12 +674,12 @@ class LexerDecoder(object):
                 i2 = int(h2, 16) - 0xDC00  # low surrogate
                 code_point = 0x10000 + (i1 << 10) + i2
 
-                part = string_ops.Utf8Encode(code_point)
+                part = Utf8Encode(code_point)
 
             elif tok_id == Id.Char_Unicode4:  # JSON only, unpaired
                 h = self.s[str_pos + 2:str_end]
                 i = int(h, 16)
-                part = string_ops.Utf8Encode(i)
+                part = Utf8Encode(i)
 
             else:
                 # Should never happen
@@ -618,11 +690,12 @@ class LexerDecoder(object):
             str_pos = str_end
 
 
-class Parser(object):
+class _Parser(object):
 
     def __init__(self, s, is_j8):
         # type: (str, bool) -> None
         self.s = s
+        self.is_j8 = is_j8
         self.lang_str = "J8" if is_j8 else "JSON"
 
         self.lexer = LexerDecoder(s, is_j8)
@@ -638,8 +711,11 @@ class Parser(object):
         while True:
             self.start_pos = self.end_pos
             self.tok_id, self.end_pos, self.decoded = self.lexer.Next()
-            if self.tok_id != Id.Ignored_Space:
+            if self.tok_id not in (Id.Ignored_Space, Id.Ignored_Comment):
                 break
+            # TODO: add Ignored_Newline to count lines, and show line numbers
+            # in errors messages.  The position of the last newline and a token
+            # can be used to calculate a column number.
 
         #log('NEXT %s %s %s %s', Id_str(self.tok_id), self.start_pos, self.end_pos, self.decoded or '-')
 
@@ -658,6 +734,14 @@ class Parser(object):
         # type: (str) -> error.Decode
         return error.Decode(msg, self.s, self.start_pos, self.end_pos)
 
+
+class Parser(_Parser):
+    """JSON and JSON8 Parser."""
+
+    def __init__(self, s, is_j8):
+        # type: (str, bool) -> None
+        _Parser.__init__(self, s, is_j8)
+
     def _ParsePair(self):
         # type: () -> Tuple[str, value_t]
 
@@ -733,14 +817,10 @@ class Parser(object):
     def _ParseValue(self):
         # type: () -> value_t
         if self.tok_id == Id.J8_LBrace:
-            d = self._ParseDict()
-            #self._Next()
-            return d
+            return self._ParseDict()
 
         elif self.tok_id == Id.J8_LBracket:
-            li = self._ParseList()
-            #self._Next()
-            return li
+            return self._ParseList()
 
         elif self.tok_id == Id.J8_Null:
             self._Next()
@@ -755,7 +835,7 @@ class Parser(object):
         elif self.tok_id == Id.J8_Int:
             part = self.s[self.start_pos:self.end_pos]
             self._Next()
-            return value.Int(int(part))
+            return value.Int(mops.FromStr(part))
 
         elif self.tok_id == Id.J8_Float:
             part = self.s[self.start_pos:self.end_pos]
@@ -781,4 +861,193 @@ class Parser(object):
         # type: () -> value_t
         """ Raises error.Decode. """
         self._Next()
-        return self._ParseValue()
+        obj = self._ParseValue()
+        if self.tok_id != Id.Eol_Tok:
+            raise self._Error('Unexpected trailing input')
+        return obj
+
+
+class Nil8Parser(_Parser):
+    """
+    Tokens not in JSON8:
+      LParen RParen Symbol
+
+    Tokens not in JSON, but in JSON8 and NIL8:
+      Identifier (unquoted keys)
+      Ignored_Comment
+    """
+
+    def __init__(self, s, is_j8):
+        # type: (str, bool) -> None
+        _Parser.__init__(self, s, is_j8)
+
+    if 0:
+
+        def _LookAhead(self):
+            # type: () -> Id_t
+            """
+            Don't need this right now
+            """
+            end_pos = self.end_pos  # look ahead from last token
+            while True:
+                tok_id, end_pos = match.MatchJ8Token(self.s, end_pos)
+                if tok_id not in (Id.Ignored_Space, Id.Ignored_Comment):
+                    break
+            return tok_id
+
+    def _ParseRecord(self):
+        # type: () -> nvalue_t
+        """
+        Yaks
+          (self->Next)             =>  (-> self Next)
+          (self->Next obj.field)   =>  ((-> self Next) (. obj field))
+
+          Similar to
+          ((identity identity) 42) => 42 in Clojure
+
+        ASDL
+          (Node left:(. x4beef2))
+          (Node left !x4beef2)
+
+        # Ambiguous because value can be identifier.
+        # We have to look ahead to and see if there's a colon :
+        field = 
+          Identifier ':' value
+        | value
+
+        record = '(' head field* ')'
+
+        - Identifier | Symbol are treated the same, it's a side effect of
+          the lexing style
+        - do positional args come before named args
+        - () is invalid?  Use [] for empty list
+        """
+        assert self.tok_id == Id.J8_LParen, Id_str(self.tok_id)
+
+        items = []  # type: List[nvalue_t]
+
+        self._Next()
+        if self.tok_id == Id.J8_RParen:
+            self._Next()
+            return nvalue.List(items)
+
+        #log('TOK %s', Id_str(self.tok_id))
+        while self.tok_id != Id.J8_RParen:
+            items.append(self._ParseNil8())
+            #log('TOK 2 %s', Id_str(self.tok_id))
+
+        self._Eat(Id.J8_RParen)
+
+        return nvalue.List(items)
+
+    def _ParseList8(self):
+        # type: () -> nvalue_t
+        """
+        List8 = '[' value* ']'
+
+        No commas, not even optional ones for now.
+        """
+        assert self.tok_id == Id.J8_LBracket, Id_str(self.tok_id)
+
+        items = []  # type: List[nvalue_t]
+
+        self._Next()
+        if self.tok_id == Id.J8_RBracket:
+            self._Next()
+            return nvalue.List(items)
+
+        #log('TOK %s', Id_str(self.tok_id))
+        while self.tok_id != Id.J8_RBracket:
+            items.append(self._ParseNil8())
+            #log('TOK 2 %s', Id_str(self.tok_id))
+
+        self._Eat(Id.J8_RBracket)
+
+        return nvalue.List(items)
+
+    def _ParseNil8(self):
+        # type: () -> nvalue_t
+        if self.tok_id == Id.J8_LParen:
+            obj = self._ParseRecord()  # type: nvalue_t
+            #return obj
+
+        elif self.tok_id == Id.J8_LBracket:
+            obj = self._ParseList8()
+            #return obj
+
+        # Primitives are copied from J8 above.
+        # TODO: We also want hex literals.
+        elif self.tok_id == Id.J8_Null:
+            self._Next()
+            obj = nvalue.Null
+
+        elif self.tok_id == Id.J8_Bool:
+            b = nvalue.Bool(self.s[self.start_pos] == 't')
+            self._Next()
+            obj = b
+
+        elif self.tok_id == Id.J8_Int:
+            part = self.s[self.start_pos:self.end_pos]
+            self._Next()
+            obj = nvalue.Int(int(part))
+
+        elif self.tok_id == Id.J8_Float:
+            part = self.s[self.start_pos:self.end_pos]
+            self._Next()
+            obj = nvalue.Float(float(part))
+
+        elif self.tok_id == Id.J8_String:
+            str_val = nvalue.Str(self.decoded)
+            self._Next()
+            obj = str_val
+
+        # <- etc.
+        elif self.tok_id in (Id.J8_Identifier, Id.J8_Operator, Id.J8_Colon,
+                             Id.J8_Comma):
+            # unquoted "word" treated like a string
+            part = self.s[self.start_pos:self.end_pos]
+            self._Next()
+            obj = nvalue.Symbol(part)
+
+        elif self.tok_id == Id.Eol_Tok:
+            raise self._Error('Unexpected EOF while parsing %s' %
+                              self.lang_str)
+
+        else:  # Id.Unknown_Tok, Id.J8_{LParen,RParen}
+            raise self._Error('Invalid token while parsing %s: %s' %
+                              (self.lang_str, Id_str(self.tok_id)))
+
+        #log('YO %s', Id_str(self.tok_id))
+        if self.tok_id in (Id.J8_Operator, Id.J8_Colon, Id.J8_Comma):
+            #log('AT %s', Id_str(self.tok_id))
+
+            # key: "value" -> (: key "value")
+            part = self.s[self.start_pos:self.end_pos]
+            op = nvalue.Symbol(part)
+
+            self._Next()
+            operand2 = self._ParseNil8()
+            infix = nvalue.List([op, obj, operand2])  # type: nvalue_t
+            #print("--> INFIX %d %s" % (id(infix), infix))
+            return infix
+
+        #next_id = self._LookAhead()
+        #print('NEXT %s' % Id_str(next_id))
+
+        #raise AssertionError()
+        #print("--> OBJ %d %s" % (id(obj), obj))
+        return obj
+
+    def ParseNil8(self):
+        # type: () -> nvalue_t
+        """ Raises error.Decode. """
+        self._Next()
+        #print('yo')
+        obj = self._ParseNil8()
+        #print("==> %d %s" % (id(obj), obj))
+        if self.tok_id != Id.Eol_Tok:
+            raise self._Error('Unexpected trailing input')
+        return obj
+
+
+# vim: sw=4
