@@ -62,7 +62,7 @@ from posix_ import (
     O_TRUNC,
 )
 
-from typing import List, Tuple, Dict, Optional, Any, cast, TYPE_CHECKING
+from typing import IO, List, Tuple, Dict, Optional, Any, cast, TYPE_CHECKING
 
 if TYPE_CHECKING:
     from _devbuild.gen.runtime_asdl import cmd_value
@@ -211,19 +211,22 @@ class FdState(object):
           IOError or OSError if the path can't be found.  (This is Python-induced wart)
         """
         fd_mode = O_RDONLY
-        return self._Open(path, 'r', fd_mode)
+        f = self._Open(path, 'r', fd_mode)
+
+        # Hacky downcast
+        return cast('mylib.LineReader', f)
 
     # used for util.DebugFile
     def OpenForWrite(self, path):
         # type: (str) -> mylib.Writer
         fd_mode = O_CREAT | O_RDWR
         f = self._Open(path, 'w', fd_mode)
-        # Hack to change mylib.LineReader into mylib.Writer.  In reality the file
-        # object supports both interfaces.
+
+        # Hacky downcast
         return cast('mylib.Writer', f)
 
     def _Open(self, path, c_mode, fd_mode):
-        # type: (str, str, int) -> mylib.LineReader
+        # type: (str, str, int) -> IO[str]
         fd = posix.open(path, fd_mode, 0o666)  # may raise OSError
 
         # Immediately move it to a new location
@@ -485,8 +488,8 @@ class FdState(object):
                     posix.write(write_fd, arg.body)
                     posix.close(write_fd)
 
-    def Push(self, redirects):
-        # type: (List[RedirValue]) -> bool
+    def Push(self, redirects, err_out):
+        # type: (List[RedirValue], List[error.IOError_OSError]) -> None
         """Apply a group of redirects and remember to undo them."""
 
         #log('> fd_state.Push %s', redirects)
@@ -500,10 +503,10 @@ class FdState(object):
                 try:
                     self._ApplyRedirect(r)
                 except (IOError, OSError) as e:
-                    self.Pop()
-                    return False  # for bad descriptor, etc.
-        #log('done applying %d redirects', len(redirects))
-        return True
+                    err_out.append(e)
+                    # This can fail too
+                    self.Pop(err_out)
+                    return  # for bad descriptor, etc.
 
     def PushStdinFromPipe(self, r):
         # type: (int) -> bool
@@ -521,8 +524,8 @@ class FdState(object):
         self._PushDup(r, redir_loc.Fd(0))
         return True
 
-    def Pop(self):
-        # type: () -> None
+    def Pop(self, err_out):
+        # type: (List[error.IOError_OSError]) -> None
         frame = self.stack.pop()
         #log('< Pop %s', frame)
         for rf in reversed(frame.saved):
@@ -531,18 +534,20 @@ class FdState(object):
                 try:
                     posix.close(rf.orig_fd)
                 except (IOError, OSError) as e:
+                    err_out.append(e)
                     log('Error closing descriptor %d: %s', rf.orig_fd,
                         pyutil.strerror(e))
-                    raise
+                    return
             else:
                 try:
                     posix.dup2(rf.saved_fd, rf.orig_fd)
                 except (IOError, OSError) as e:
+                    err_out.append(e)
                     log('dup2(%d, %d) error: %s', rf.saved_fd, rf.orig_fd,
                         pyutil.strerror(e))
                     #log('fd state:')
                     #posix.system('ls -l /proc/%s/fd' % posix.getpid())
-                    raise
+                    return
                 posix.close(rf.saved_fd)
                 #log('dup2 %s %s', saved, orig)
 
@@ -619,17 +624,18 @@ OWN_LEADER = 0
 
 class SetPgid(ChildStateChange):
 
-    def __init__(self, pgid):
-        # type: (int) -> None
+    def __init__(self, pgid, tracer):
+        # type: (int, dev.Tracer) -> None
         self.pgid = pgid
+        self.tracer = tracer
 
     def Apply(self):
         # type: () -> None
         try:
             posix.setpgid(0, self.pgid)
         except (IOError, OSError) as e:
-            print_stderr(
-                'osh: child failed to set process group for PID %d to %d: %s' %
+            self.tracer.OtherMessage(
+                'osh: child %d failed to set its process group to %d: %s' %
                 (posix.getpid(), self.pgid, pyutil.strerror(e)))
 
     def ApplyFromParent(self, proc):
@@ -637,7 +643,7 @@ class SetPgid(ChildStateChange):
         try:
             posix.setpgid(proc.pid, self.pgid)
         except (IOError, OSError) as e:
-            print_stderr(
+            self.tracer.OtherMessage(
                 'osh: parent failed to set process group for PID %d to %d: %s'
                 % (proc.pid, self.pgid, pyutil.strerror(e)))
 
@@ -654,10 +660,10 @@ class ExternalProgram(object):
     ):
         # type: (...) -> None
         """
-    Args:
-      hijack_shebang: The path of an interpreter to run instead of the one
-        specified in the shebang line.  May be empty.
-    """
+        Args:
+          hijack_shebang: The path of an interpreter to run instead of the one
+            specified in the shebang line.  May be empty.
+        """
         self.hijack_shebang = hijack_shebang
         self.fd_state = fd_state
         self.errfmt = errfmt
@@ -834,7 +840,8 @@ class SubProgramThunk(Thunk):
             print('')
             status = 130  # 128 + 2
         except (IOError, OSError) as e:
-            print_stderr('osh I/O error (subprogram): %s' % pyutil.strerror(e))
+            print_stderr('oils I/O error (subprogram): %s' %
+                         pyutil.strerror(e))
             status = 2
 
         # If ProcessInit() doesn't turn off buffering, this is needed before
@@ -943,10 +950,10 @@ class Process(Job):
     def __init__(self, thunk, job_control, job_list, tracer):
         # type: (Thunk, JobControl, JobList, dev.Tracer) -> None
         """
-    Args:
-      thunk: Thunk instance
-      job_list: for process bookkeeping
-    """
+        Args:
+          thunk: Thunk instance
+          job_list: for process bookkeeping
+        """
         Job.__init__(self)
         assert isinstance(thunk, Thunk), thunk
         self.thunk = thunk
@@ -1155,10 +1162,11 @@ class Process(Job):
 
 class ctx_Pipe(object):
 
-    def __init__(self, fd_state, fd):
-        # type: (FdState, int) -> None
+    def __init__(self, fd_state, fd, err_out):
+        # type: (FdState, int, List[error.IOError_OSError]) -> None
         fd_state.PushStdinFromPipe(fd)
         self.fd_state = fd_state
+        self.err_out = err_out
 
     def __enter__(self):
         # type: () -> None
@@ -1166,7 +1174,7 @@ class ctx_Pipe(object):
 
     def __exit__(self, type, value, traceback):
         # type: (Any, Any, Any) -> None
-        self.fd_state.Pop()
+        self.fd_state.Pop(self.err_out)
 
 
 class Pipeline(Job):
@@ -1179,16 +1187,20 @@ class Pipeline(Job):
     foo | bar | read v
     """
 
-    def __init__(self, sigpipe_status_ok, job_control, job_list):
-        # type: (bool, JobControl, JobList) -> None
+    def __init__(self, sigpipe_status_ok, job_control, job_list, tracer):
+        # type: (bool, JobControl, JobList, dev.Tracer) -> None
         Job.__init__(self)
         self.job_control = job_control
         self.job_list = job_list
+        self.tracer = tracer
+
         self.procs = []  # type: List[Process]
         self.pids = []  # type: List[int]  # pids in order
         self.pipe_status = []  # type: List[int]  # status in order
         self.status = -1  # for 'wait' jobs
 
+        self.pgid = INVALID_PGID
+
         # Optional for foreground
         self.last_thunk = None  # type: Tuple[CommandEvaluator, command_t]
         self.last_pipe = None  # type: Tuple[int, int]
@@ -1198,9 +1210,7 @@ class Pipeline(Job):
     def ProcessGroupId(self):
         # type: () -> int
         """Returns the group ID of this pipeline."""
-        # This should only ever be called AFTER the pipeline has started
-        assert len(self.pids) > 0
-        return self.pids[0]  # First process is the group leader
+        return self.pgid
 
     def DisplayJob(self, job_id, f, style):
         # type: (int, mylib.Writer, int) -> None
@@ -1270,19 +1280,19 @@ class Pipeline(Job):
         # If we are creating a pipeline in a subshell or we aren't running with job
         # control, our children should remain in our inherited process group.
         # the pipelines's group ID.
-        pgid = INVALID_PGID
         if self.job_control.Enabled():
-            pgid = OWN_LEADER  # first process in pipeline is the leader
+            self.pgid = OWN_LEADER  # first process in pipeline is the leader
 
         for i, proc in enumerate(self.procs):
-            if pgid != INVALID_PGID:
-                proc.AddStateChange(SetPgid(pgid))
+            if self.pgid != INVALID_PGID:
+                proc.AddStateChange(SetPgid(self.pgid, self.tracer))
 
+            # Figure out the pid
             pid = proc.StartProcess(trace.PipelinePart)
-            if i == 0 and pgid != INVALID_PGID:
-                # Mimic bash and use the PID of the first process as the group for the
+            if i == 0 and self.pgid != INVALID_PGID:
+                # Mimic bash and use the PID of the FIRST process as the group for the
                 # whole pipeline.
-                pgid = pid
+                self.pgid = pid
 
             self.pids.append(pid)
             self.pipe_status.append(-1)  # uninitialized
@@ -1341,7 +1351,13 @@ class Pipeline(Job):
         """
         assert len(self.pids) == len(self.procs)
 
-        self.job_control.MaybeGiveTerminal(posix.getpgid(self.pids[0]))
+        # TODO: break circular dep.  Bit flags could go in ASDL or headers.
+        from osh import cmd_eval
+
+        # This is tcsetpgrp()
+        # TODO: fix race condition -- I believe the first process could have
+        # stopped already, and thus getpgid() will fail
+        self.job_control.MaybeGiveTerminal(self.pgid)
 
         # Run the last part of the pipeline IN PARALLEL with other processes.  It
         # may or may not fork:
@@ -1354,8 +1370,15 @@ class Pipeline(Job):
         r, w = self.last_pipe  # set in AddLast()
         posix.close(w)  # we will not write here
 
-        with ctx_Pipe(fd_state, r):
-            cmd_ev.ExecuteAndCatch(last_node)
+        # Fix lastpipe / job control / DEBUG trap interaction
+        cmd_flags = cmd_eval.NoDebugTrap if self.job_control.Enabled() else 0
+        io_errors = []  # type: List[error.IOError_OSError]
+        with ctx_Pipe(fd_state, r, io_errors):
+            cmd_ev.ExecuteAndCatch(last_node, cmd_flags)
+
+        if len(io_errors):
+            e_die('Error setting up last part of pipeline: %s' %
+                  pyutil.strerror(io_errors[0]))
 
         # We won't read anymore.  If we don't do this, then 'cat' in 'cat
         # /dev/urandom | sleep 1' will never get SIGPIPE.
@@ -1491,6 +1514,9 @@ class JobControl(object):
 
     def Enabled(self):
         # type: () -> bool
+
+        # TODO: get rid of this syscall?  SubProgramThunk should set a flag I
+        # think.
         curr_pid = posix.getpid()
         # Only the main shell should bother with job control functions.
         return curr_pid == self.shell_pid and self.shell_tty_fd != -1
