@@ -5,6 +5,7 @@ word_eval.py - Evaluator for the word language.
 from _devbuild.gen.id_kind_asdl import Id, Kind, Kind_str
 from _devbuild.gen.syntax_asdl import (
     Token,
+    NameTok,
     loc,
     loc_t,
     BracedVarSub,
@@ -16,7 +17,6 @@ from _devbuild.gen.syntax_asdl import (
     ShArrayLiteral,
     SingleQuoted,
     DoubleQuoted,
-    SimpleVarSub,
     word_e,
     word_t,
     CompoundWord,
@@ -58,6 +58,7 @@ from core.error import e_die
 from frontend import consts
 from frontend import lexer
 from frontend import location
+from mycpp import mops
 from mycpp.mylib import log, tagswitch, NewDict
 from osh import braces
 from osh import glob_
@@ -475,10 +476,12 @@ class TildeEvaluator(object):
 
         if result is None:
             if self.exec_opts.strict_tilde():
-                e_die("Error expanding tilde (e.g. invalid user)", part.token)
+                e_die("Error expanding tilde (e.g. invalid user)", part.left)
             else:
                 # Return ~ or ~user literally
-                return lexer.TokenVal(part.token)
+                result = '~'
+                if part.user_name is not None:
+                    result = result + part.user_name  # mycpp doesn't have +=
 
         return result
 
@@ -1454,7 +1457,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
         return self._ConcatPartVals(part_vals, part.left)
 
     def _EvalSimpleVarSub(self, part, part_vals, quoted):
-        # type: (SimpleVarSub, List[part_value_t], bool) -> None
+        # type: (NameTok, List[part_value_t], bool) -> None
 
         # TODO: use name
         token = part.left
@@ -1496,7 +1499,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
         part_vals.append(v)
 
     def EvalSimpleVarSubToString(self, node):
-        # type: (SimpleVarSub) -> str
+        # type: (NameTok) -> str
         """For double quoted strings in YSH expressions.
 
         Example: var x = "$foo-${foo}"
@@ -1627,7 +1630,7 @@ class AbstractWordEvaluator(StringWordEvaluator):
                 part_vals.append(sv)
 
             elif case(word_part_e.SimpleVarSub):
-                part = cast(SimpleVarSub, UP_part)
+                part = cast(NameTok, UP_part)
                 self._EvalSimpleVarSub(part, part_vals, quoted)
 
             elif case(word_part_e.BracedVarSub):
@@ -1645,8 +1648,8 @@ class AbstractWordEvaluator(StringWordEvaluator):
 
             elif case(word_part_e.ArithSub):
                 part = cast(word_part.ArithSub, UP_part)
-                num = self.arith_ev.EvalToInt(part.anode)
-                v = part_value.String(str(num), quoted, not quoted)
+                num = self.arith_ev.EvalToBigInt(part.anode)
+                v = part_value.String(mops.ToStr(num), quoted, not quoted)
                 part_vals.append(v)
 
             elif case(word_part_e.ExtGlob):
