@@ -54,11 +54,11 @@ from _devbuild.gen.types_asdl import (lex_mode_t, lex_mode_e)
 from _devbuild.gen.syntax_asdl import (
     BoolParamBox,
     Token,
+    NameTok,
     loc,
     source,
     DoubleQuoted,
     SingleQuoted,
-    SimpleVarSub,
     BracedVarSub,
     CommandSub,
     ShArrayLiteral,
@@ -75,7 +75,6 @@ from _devbuild.gen.syntax_asdl import (
     CompoundWord,
     word_part,
     word_part_t,
-    y_lhs,
     y_lhs_e,
     arith_expr_t,
     command,
@@ -197,8 +196,6 @@ class WordParser(WordEmitter):
             self.parse_ctx.trail.AppendToken(self.cur_token)  # For completion
             self.next_lex_mode = lex_mode_e.Undefined
 
-            #log('_GetToken %s %r', Id_str(self.token_type), self.cur_token.tval)
-
     def _SetNext(self, lex_mode):
         # type: (lex_mode_t) -> None
         """Set the next lex state, but don't actually read a token.
@@ -313,6 +310,7 @@ class WordParser(WordEmitter):
             # read until }
             replace = self._ReadVarOpArg(
                 lex_mode_e.VSub_ArgUnquoted)  # type: rhs_word_t
+            #log('r 1 %r', replace)
         else:
             # e.g. ${v/a} is the same as ${v/a/}  -- empty replacement string
             replace = rhs_word.Empty
@@ -434,11 +432,8 @@ class WordParser(WordEmitter):
 
         elif op_kind == Kind.VOp2:  # / : [ ]
             if self.token_type == Id.VOp2_Slash:
-                patsub_op = self._ReadPatSubVarOp()
-
-                # awkwardness for mycpp; could fix
-                temp = cast(suffix_op_t, patsub_op)
-                part.suffix_op = temp
+                patsub_op = self._ReadPatSubVarOp()  # type: suffix_op_t
+                part.suffix_op = patsub_op
 
                 # Checked by the method above
                 assert self.token_type == Id.Right_DollarBrace, self.cur_token
@@ -621,7 +616,10 @@ class WordParser(WordEmitter):
 
     def ReadSingleQuoted(self, lex_mode, left_token, tokens, is_ysh_expr):
         # type: (lex_mode_t, Token, List[Token], bool) -> Token
-        """Used by expr_parse.py."""
+        """Appends to tokens
+
+        Used by expr_parse.py
+        """
 
         # echo '\' is allowed, but x = '\' is invalid, in favor of x = r'\'
         no_backslashes = is_ysh_expr and left_token.id == Id.Left_SingleQuote
@@ -667,7 +665,9 @@ class WordParser(WordEmitter):
 
                 # x = $'\z' is disallowed; ditto for echo $'\z' if shopt -u parse_backslash
                 if is_ysh_expr or not self.parse_opts.parse_backslash():
-                    p_die("Invalid char escape in C-style string literal", tok)
+                    p_die(
+                        "Invalid char escape in C-style string literal (OILS-ERR-11)",
+                        tok)
 
                 tokens.append(tok)
 
@@ -944,7 +944,7 @@ class WordParser(WordEmitter):
                         if (is_ysh_expr or
                                 not self.parse_opts.parse_backslash()):
                             p_die(
-                                "Invalid char escape in double quoted string",
+                                "Invalid char escape in double quoted string (OILS-ERR-12)",
                                 self.cur_token)
                     elif self.token_type == Id.Lit_Dollar:
                         if is_ysh_expr or not self.parse_opts.parse_dollar():
@@ -964,7 +964,7 @@ class WordParser(WordEmitter):
 
             elif self.token_kind == Kind.VSub:
                 tok = self.cur_token
-                part = SimpleVarSub(tok, lexer.TokenSliceLeft(tok, 1))
+                part = NameTok(tok, lexer.TokenSliceLeft(tok, 1))
                 out_parts.append(part)
                 # NOTE: parsing "$f(x)" would BREAK CODE.  Could add a more for it
                 # later.
@@ -1057,7 +1057,7 @@ class WordParser(WordEmitter):
 
             right_token = c_parser.w_parser.cur_token
 
-        elif left_id == Id.Left_Backtick and self.parse_ctx.one_pass_parse:
+        elif left_id == Id.Left_Backtick and self.parse_ctx.do_lossless:
             # NOTE: This is an APPROXIMATE solution for translation ONLY.  See
             # test/osh2oil.
 
@@ -1081,7 +1081,8 @@ class WordParser(WordEmitter):
                 #log("TOK %s", self.cur_token)
 
                 if self.token_type == Id.Backtick_Quoted:
-                    parts.append(self.cur_token.tval[1:])  # Remove leading \
+                    # Remove leading \
+                    parts.append(lexer.TokenSliceLeft(self.cur_token, 1))
 
                 elif self.token_type == Id.Backtick_DoubleQuote:
                     # Compatibility: If backticks are double quoted, then double quotes
@@ -1089,13 +1090,13 @@ class WordParser(WordEmitter):
                     # Shells aren't smart enough to match nested " and ` quotes (but OSH
                     # is)
                     if d_quoted:
-                        parts.append(
-                            self.cur_token.tval[1:])  # Remove leading \
+                        # Remove leading \
+                        parts.append(lexer.TokenSliceLeft(self.cur_token, 1))
                     else:
-                        parts.append(self.cur_token.tval)
+                        parts.append(lexer.TokenVal(self.cur_token))
 
                 elif self.token_type == Id.Backtick_Other:
-                    parts.append(self.cur_token.tval)
+                    parts.append(lexer.TokenVal(self.cur_token))
 
                 elif self.token_type == Id.Backtick_Right:
                     break
@@ -1188,8 +1189,8 @@ class WordParser(WordEmitter):
             UP_lhs = lhs
             with tagswitch(lhs) as case:
                 if case(y_lhs_e.Var):
-                    lhs = cast(y_lhs.Var, UP_lhs)
-                    var_checker.Check(kw_token.id, lhs.name)
+                    lhs = cast(NameTok, UP_lhs)
+                    var_checker.Check(kw_token.id, lhs.var_name, lhs.left)
 
                 # Note: this does not cover cases like
                 # setvar (a[0])[1] = v
@@ -1199,14 +1200,14 @@ class WordParser(WordEmitter):
                 elif case(y_lhs_e.Subscript):
                     lhs = cast(Subscript, UP_lhs)
                     if lhs.obj.tag() == expr_e.Var:
-                        var_checker.Check(kw_token.id,
-                                          cast(expr.Var, lhs.obj).name)
+                        v = cast(expr.Var, lhs.obj)
+                        var_checker.Check(kw_token.id, v.name, v.left)
 
                 elif case(y_lhs_e.Attribute):
                     lhs = cast(Attribute, UP_lhs)
                     if lhs.obj.tag() == expr_e.Var:
-                        var_checker.Check(kw_token.id,
-                                          cast(expr.Var, lhs.obj).name)
+                        v = cast(expr.Var, lhs.obj)
+                        var_checker.Check(kw_token.id, v.name, v.left)
 
         # Let the CommandParser see the Op_Semi or Op_Newline.
         self.buffered_word = last_token
@@ -1611,7 +1612,7 @@ class WordParser(WordEmitter):
             ch = lexer.TokenSliceLeft(tok, 1)
             if not self.parse_opts.parse_backslash():
                 if not pyutil.IsValidCharEscape(ch):
-                    p_die('Invalid char escape (parse_backslash)',
+                    p_die('Invalid char escape in unquoted word (OILS-ERR-13)',
                           self.cur_token)
 
             part = word_part.EscapedLiteral(self.cur_token,
@@ -1743,9 +1744,9 @@ class WordParser(WordEmitter):
             elif self.token_kind == Kind.VSub:
                 vsub_token = self.cur_token
 
-                part = SimpleVarSub(vsub_token,
-                                    lexer.TokenSliceLeft(
-                                        vsub_token, 1))  # type: word_part_t
+                part = NameTok(vsub_token,
+                               lexer.TokenSliceLeft(vsub_token,
+                                                    1))  # type: word_part_t
                 w.parts.append(part)
 
             elif self.token_kind == Kind.ExtGlob:
